#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2007-01-18 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

# FIX: check to see if these will be needed
TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


fieldList = [
	'MessageID',
	'RepeatIndicator',
	'UserID',
	'NavigationStatus',
	'ROT',
	'SOG',
	'PositionAccuracy',
	'Position_longitude',
	'Position_latitude',
	'COG',
	'TrueHeading',
	'TimeStamp',
	'RegionalReserved',
	'Spare',
	'RAIM',
	'syncstate',
	'slotoffset',
]

def encode(params, validate=False):
	'''Create a position binary message payload to pack into an AIS Msg position.

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 1 (field automatically set to "1")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - NavigationStatus(uint): What is the vessel doing
	  - ROT(int): RateOfTurn
	  - SOG(udecimal): Speed over ground
	  - PositionAccuracy(uint): Accuracy of positioning fixes
	  - Position_longitude(decimal): Location of the vessel  East West location
	  - Position_latitude(decimal): Location of the vessel  North South location
	  - COG(udecimal): Course over ground
	  - TrueHeading(uint): True heading (relative to true North)
	  - TimeStamp(uint): UTC second when the report was generated
	  - RegionalReserved(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - RAIM(bool): Receiver autonomous integrity monitoring flag
	  - syncstate(uint): Sycronization state
	  - slotoffset(uint): In what slot will the next transmission occur. BROKEN
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	@note: The returned bits may not be 6 bit aligned.  It is up to you to pad out the bits.
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=1),6))
	if 'RepeatIndicator' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['RepeatIndicator']),2))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=0),2))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['UserID']),30))
	if 'NavigationStatus' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['NavigationStatus']),4))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=15),4))
	if 'ROT' in params:
		bvList.append(binary.bvFromSignedInt(params['ROT'],8))
	else:
		bvList.append(binary.bvFromSignedInt(-128,8))
	if 'SOG' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['SOG'])*Decimal('10')))),10))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(1023)),10))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['PositionAccuracy']),1))
	if 'Position_longitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['Position_longitude'])*Decimal('600000')),28))
	else:
		bvList.append(binary.bvFromSignedInt(108600000,28))
	if 'Position_latitude' in params:
		bvList.append(binary.bvFromSignedInt(int(Decimal(params['Position_latitude'])*Decimal('600000')),27))
	else:
		bvList.append(binary.bvFromSignedInt(54600000,27))
	if 'COG' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int((Decimal(params['COG'])*Decimal('10')))),12))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=int(3600)),12))
	if 'TrueHeading' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['TrueHeading']),9))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=511),9))
	if 'TimeStamp' in params:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=params['TimeStamp']),6))
	else:
		bvList.append(binary.setBitVectorSize(BitVector(intVal=60),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),4))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),1))
	if params["RAIM"]: bvList.append(TrueBV)
	else: bvList.append(FalseBV)
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['syncstate']),2))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['slotoffset']),14))

	return binary.joinBV(bvList)

def decode(bv, validate=False):
	'''Unpack a position message 

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 1 (field automatically set to "1")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - NavigationStatus(uint): What is the vessel doing
	  - ROT(int): RateOfTurn
	  - SOG(udecimal): Speed over ground
	  - PositionAccuracy(uint): Accuracy of positioning fixes
	  - Position_longitude(decimal): Location of the vessel  East West location
	  - Position_latitude(decimal): Location of the vessel  North South location
	  - COG(udecimal): Course over ground
	  - TrueHeading(uint): True heading (relative to true North)
	  - TimeStamp(uint): UTC second when the report was generated
	  - RegionalReserved(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - RAIM(bool): Receiver autonomous integrity monitoring flag
	  - syncstate(uint): Sycronization state
	  - slotoffset(uint): In what slot will the next transmission occur. BROKEN
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['MessageID']=1
	r['RepeatIndicator']=int(bv[6:8])
	r['UserID']=int(bv[8:38])
	r['NavigationStatus']=int(bv[38:42])
	r['ROT']=binary.signedIntFromBV(bv[42:50])
	r['SOG']=Decimal(int(bv[50:60]))/Decimal('10')
	r['PositionAccuracy']=int(bv[60:61])
	r['Position_longitude']=Decimal(binary.signedIntFromBV(bv[61:89]))/Decimal('600000')
	r['Position_latitude']=Decimal(binary.signedIntFromBV(bv[89:116]))/Decimal('600000')
	r['COG']=Decimal(int(bv[116:128]))/Decimal('10')
	r['TrueHeading']=int(bv[128:137])
	r['TimeStamp']=int(bv[137:143])
	r['RegionalReserved']=0
	r['Spare']=0
	r['RAIM']=bool(int(bv[148:149]))
	r['syncstate']=int(bv[149:151])
	r['slotoffset']=int(bv[151:165])
	return r

def decodeMessageID(bv, validate=False):
	return 1

def decodeRepeatIndicator(bv, validate=False):
	return int(bv[6:8])

def decodeUserID(bv, validate=False):
	return int(bv[8:38])

def decodeNavigationStatus(bv, validate=False):
	return int(bv[38:42])

def decodeROT(bv, validate=False):
	return binary.signedIntFromBV(bv[42:50])

def decodeSOG(bv, validate=False):
	return Decimal(int(bv[50:60]))/Decimal('10')

def decodePositionAccuracy(bv, validate=False):
	return int(bv[60:61])

def decodePosition_longitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[61:89]))/Decimal('600000')

def decodePosition_latitude(bv, validate=False):
	return Decimal(binary.signedIntFromBV(bv[89:116]))/Decimal('600000')

def decodeCOG(bv, validate=False):
	return Decimal(int(bv[116:128]))/Decimal('10')

def decodeTrueHeading(bv, validate=False):
	return int(bv[128:137])

def decodeTimeStamp(bv, validate=False):
	return int(bv[137:143])

def decodeRegionalReserved(bv, validate=False):
	return 0

def decodeSpare(bv, validate=False):
	return 0

def decodeRAIM(bv, validate=False):
	return bool(int(bv[148:149]))

def decodesyncstate(bv, validate=False):
	return int(bv[149:151])

def decodeslotoffset(bv, validate=False):
	return int(bv[151:165])


def printHtml(params, out=sys.stdout):
		out.write("<h3>position<h3>\n")
		out.write("<table border=\"1\">\n")
		out.write("<tr bgcolor=\"orange\">\n")
		out.write("<th align=\"left\">Field Name</th>\n")
		out.write("<th align=\"left\">Type</th>\n")
		out.write("<th align=\"left\">Value</th>\n")
		out.write("<th align=\"left\">Value in Lookup Table</th>\n")
		out.write("<th align=\"left\">Units</th>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>MessageID</td>\n")
		out.write("<td>uint</td>\n")
		if 'MessageID' in params:
			out.write("	<td>"+str(params['MessageID'])+"</td>\n")
			out.write("	<td>"+str(params['MessageID'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>RepeatIndicator</td>\n")
		out.write("<td>uint</td>\n")
		if 'RepeatIndicator' in params:
			out.write("	<td>"+str(params['RepeatIndicator'])+"</td>\n")
			if str(params['RepeatIndicator']) in RepeatIndicatorDecodeLut:
				out.write("<td>"+RepeatIndicatorDecodeLut[str(params['RepeatIndicator'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>UserID</td>\n")
		out.write("<td>uint</td>\n")
		if 'UserID' in params:
			out.write("	<td>"+str(params['UserID'])+"</td>\n")
			out.write("	<td>"+str(params['UserID'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>NavigationStatus</td>\n")
		out.write("<td>uint</td>\n")
		if 'NavigationStatus' in params:
			out.write("	<td>"+str(params['NavigationStatus'])+"</td>\n")
			if str(params['NavigationStatus']) in NavigationStatusDecodeLut:
				out.write("<td>"+NavigationStatusDecodeLut[str(params['NavigationStatus'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>ROT</td>\n")
		out.write("<td>int</td>\n")
		if 'ROT' in params:
			out.write("	<td>"+str(params['ROT'])+"</td>\n")
			out.write("	<td>"+str(params['ROT'])+"</td>\n")
		out.write("<td>4.733*sqrt(val) degrees/min</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>SOG</td>\n")
		out.write("<td>udecimal</td>\n")
		if 'SOG' in params:
			out.write("	<td>"+str(params['SOG'])+"</td>\n")
			if str(params['SOG']) in SOGDecodeLut:
				out.write("<td>"+SOGDecodeLut[str(params['SOG'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("<td>knots</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>PositionAccuracy</td>\n")
		out.write("<td>uint</td>\n")
		if 'PositionAccuracy' in params:
			out.write("	<td>"+str(params['PositionAccuracy'])+"</td>\n")
			if str(params['PositionAccuracy']) in PositionAccuracyDecodeLut:
				out.write("<td>"+PositionAccuracyDecodeLut[str(params['PositionAccuracy'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>Position_longitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'Position_longitude' in params:
			out.write("	<td>"+str(params['Position_longitude'])+"</td>\n")
			out.write("	<td>"+str(params['Position_longitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>Position_latitude</td>\n")
		out.write("<td>decimal</td>\n")
		if 'Position_latitude' in params:
			out.write("	<td>"+str(params['Position_latitude'])+"</td>\n")
			out.write("	<td>"+str(params['Position_latitude'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>COG</td>\n")
		out.write("<td>udecimal</td>\n")
		if 'COG' in params:
			out.write("	<td>"+str(params['COG'])+"</td>\n")
			out.write("	<td>"+str(params['COG'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>TrueHeading</td>\n")
		out.write("<td>uint</td>\n")
		if 'TrueHeading' in params:
			out.write("	<td>"+str(params['TrueHeading'])+"</td>\n")
			out.write("	<td>"+str(params['TrueHeading'])+"</td>\n")
		out.write("<td>degrees</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>TimeStamp</td>\n")
		out.write("<td>uint</td>\n")
		if 'TimeStamp' in params:
			out.write("	<td>"+str(params['TimeStamp'])+"</td>\n")
			if str(params['TimeStamp']) in TimeStampDecodeLut:
				out.write("<td>"+TimeStampDecodeLut[str(params['TimeStamp'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("<td>seconds</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>RegionalReserved</td>\n")
		out.write("<td>uint</td>\n")
		if 'RegionalReserved' in params:
			out.write("	<td>"+str(params['RegionalReserved'])+"</td>\n")
			out.write("	<td>"+str(params['RegionalReserved'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>Spare</td>\n")
		out.write("<td>uint</td>\n")
		if 'Spare' in params:
			out.write("	<td>"+str(params['Spare'])+"</td>\n")
			out.write("	<td>"+str(params['Spare'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>RAIM</td>\n")
		out.write("<td>bool</td>\n")
		if 'RAIM' in params:
			out.write("	<td>"+str(params['RAIM'])+"</td>\n")
			if str(params['RAIM']) in RAIMDecodeLut:
				out.write("<td>"+RAIMDecodeLut[str(params['RAIM'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>syncstate</td>\n")
		out.write("<td>uint</td>\n")
		if 'syncstate' in params:
			out.write("	<td>"+str(params['syncstate'])+"</td>\n")
			if str(params['syncstate']) in syncstateDecodeLut:
				out.write("<td>"+syncstateDecodeLut[str(params['syncstate'])]+"</td>")
			else:
				out.write("<td><i>Missing LUT entry</i></td>")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>slotoffset</td>\n")
		out.write("<td>uint</td>\n")
		if 'slotoffset' in params:
			out.write("	<td>"+str(params['slotoffset'])+"</td>\n")
			out.write("	<td>"+str(params['slotoffset'])+"</td>\n")
		out.write("</tr>\n")
		out.write("</table>\n")


def printKml(params, out=sys.stdout):
	'''KML (Keyhole Markup Language) for Google Earth, but without the header/footer'''
	out.write("\	<Placemark>\n")
	out.write("\t	<name>"+str(params['UserID'])+"</name>\n")
	out.write("\t\t<description>\n")
	import StringIO
	buf = StringIO.StringIO()
	printHtml(params,buf)
	import cgi
	out.write(cgi.escape(buf.getvalue()))
	out.write("\t\t</description>\n")
	out.write("\t\t<styleUrl>#m_ylw-pushpin_copy0</styleUrl>\n")
	out.write("\t\t<Point>\n")
	out.write("\t\t\t<coordinates>")
	out.write(str(params['Position_longitude']))
	out.write(',')
	out.write(str(params['Position_latitude']))
	out.write(",0</coordinates>\n")
	out.write("\t\t</Point>\n")
	out.write("\t</Placemark>\n")

def printFields(params, out=sys.stdout, format='std', fieldList=None):
	'''Print a slotoffset message to stdout.

	Fields in params:
	  - MessageID(uint): AIS message number.  Must be 1 (field automatically set to "1")
	  - RepeatIndicator(uint): Indicated how many times a message has been repeated
	  - UserID(uint): Unique ship identification number (MMSI)
	  - NavigationStatus(uint): What is the vessel doing
	  - ROT(int): RateOfTurn
	  - SOG(udecimal): Speed over ground
	  - PositionAccuracy(uint): Accuracy of positioning fixes
	  - Position_longitude(decimal): Location of the vessel  East West location
	  - Position_latitude(decimal): Location of the vessel  North South location
	  - COG(udecimal): Course over ground
	  - TrueHeading(uint): True heading (relative to true North)
	  - TimeStamp(uint): UTC second when the report was generated
	  - RegionalReserved(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - Spare(uint): Reserved for definition by a regional authority. (field automatically set to "0")
	  - RAIM(bool): Receiver autonomous integrity monitoring flag
	  - syncstate(uint): Sycronization state
	  - slotoffset(uint): In what slot will the next transmission occur. BROKEN
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	if 'std'==format:
		out.write("slotoffset:\n")
		if 'MessageID' in params: out.write("	MessageID:           "+str(params['MessageID'])+"\n")
		if 'RepeatIndicator' in params: out.write("	RepeatIndicator:     "+str(params['RepeatIndicator'])+"\n")
		if 'UserID' in params: out.write("	UserID:              "+str(params['UserID'])+"\n")
		if 'NavigationStatus' in params: out.write("	NavigationStatus:    "+str(params['NavigationStatus'])+"\n")
		if 'ROT' in params: out.write("	ROT:                 "+str(params['ROT'])+"\n")
		if 'SOG' in params: out.write("	SOG:                 "+str(params['SOG'])+"\n")
		if 'PositionAccuracy' in params: out.write("	PositionAccuracy:    "+str(params['PositionAccuracy'])+"\n")
		if 'Position_longitude' in params: out.write("	Position_longitude:  "+str(params['Position_longitude'])+"\n")
		if 'Position_latitude' in params: out.write("	Position_latitude:   "+str(params['Position_latitude'])+"\n")
		if 'COG' in params: out.write("	COG:                 "+str(params['COG'])+"\n")
		if 'TrueHeading' in params: out.write("	TrueHeading:         "+str(params['TrueHeading'])+"\n")
		if 'TimeStamp' in params: out.write("	TimeStamp:           "+str(params['TimeStamp'])+"\n")
		if 'RegionalReserved' in params: out.write("	RegionalReserved:    "+str(params['RegionalReserved'])+"\n")
		if 'Spare' in params: out.write("	Spare:               "+str(params['Spare'])+"\n")
		if 'RAIM' in params: out.write("	RAIM:                "+str(params['RAIM'])+"\n")
		if 'syncstate' in params: out.write("	syncstate:           "+str(params['syncstate'])+"\n")
		if 'slotoffset' in params: out.write("	slotoffset:          "+str(params['slotoffset'])+"\n")
	elif 'csv'==format:
		if None == options.fieldList:
			options.fieldList = fieldList
		needComma = False;
		for field in fieldList:
			if needComma: out.write(',')
			needComma = True
			if field in params:
				out.write(str(params[field]))
			# else: leave it empty
		out.write("\n")
	elif 'html'==format:
		printHtml(params,out)
	elif 'kml'==format:
		printKml(params,out)
	elif 'kml-full'==format:
		out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
		out.write("<kml xmlns=\"http://earth.google.com/kml/2.1\">\n")
		out.write("<Document>\n")
		out.write("	<name>Position</name>\n")
		printKml(params,out)
		out.write("</Document>\n")
		out.write("</kml>\n")
	else: 
		print "ERROR: unknown format:",format
		assert False

	return # Nothing to return

RepeatIndicatorEncodeLut = {
	'default':'0',
	'do not repeat any more':'3',
	} #RepeatIndicatorEncodeLut

RepeatIndicatorDecodeLut = {
	'0':'default',
	'3':'do not repeat any more',
	} # RepeatIndicatorEncodeLut

NavigationStatusEncodeLut = {
	'under way using engine':'0',
	'at anchor':'1',
	'not under command':'2',
	'restricted maneuverability':'3',
	'constrained by her draught':'4',
	'moored':'5',
	'aground':'6',
	'engaged in fishing':'7',
	'under way sailing':'8',
	'reserved for future use (hazmat)':'9',
	'reserved for future use':'10',
	'reserved for future use':'11',
	'reserved for future use':'12',
	'reserved for future use':'13',
	'reserved for future use':'14',
	'not defined = default':'15',
	} #NavigationStatusEncodeLut

NavigationStatusDecodeLut = {
	'0':'under way using engine',
	'1':'at anchor',
	'2':'not under command',
	'3':'restricted maneuverability',
	'4':'constrained by her draught',
	'5':'moored',
	'6':'aground',
	'7':'engaged in fishing',
	'8':'under way sailing',
	'9':'reserved for future use (hazmat)',
	'10':'reserved for future use',
	'11':'reserved for future use',
	'12':'reserved for future use',
	'13':'reserved for future use',
	'14':'reserved for future use',
	'15':'not defined = default',
	} # NavigationStatusEncodeLut

SOGEncodeLut = {
	'102.2 knots or higher':'102.2',
	} #SOGEncodeLut

SOGDecodeLut = {
	'102.2':'102.2 knots or higher',
	} # SOGEncodeLut

PositionAccuracyEncodeLut = {
	'low (greater than 10 m)':'0',
	'high (less than 10 m)':'1',
	} #PositionAccuracyEncodeLut

PositionAccuracyDecodeLut = {
	'0':'low (greater than 10 m)',
	'1':'high (less than 10 m)',
	} # PositionAccuracyEncodeLut

TimeStampEncodeLut = {
	'not available/default':'60',
	'manual input':'61',
	'dead reckoning':'62',
	'inoperative':'63',
	} #TimeStampEncodeLut

TimeStampDecodeLut = {
	'60':'not available/default',
	'61':'manual input',
	'62':'dead reckoning',
	'63':'inoperative',
	} # TimeStampEncodeLut

RAIMEncodeLut = {
	'not in use':'False',
	'in use':'True',
	} #RAIMEncodeLut

RAIMDecodeLut = {
	'False':'not in use',
	'True':'in use',
	} # RAIMEncodeLut

syncstateEncodeLut = {
	'UTC direct':'0',
	'UTC indirect':'1',
	'synchronized to a base station':'2',
	'synchronized to another station':'3',
	} #syncstateEncodeLut

syncstateDecodeLut = {
	'0':'UTC direct',
	'1':'UTC indirect',
	'2':'synchronized to a base station',
	'3':'synchronized to another station',
	} # syncstateEncodeLut



######################################################################
# UNIT TESTING
######################################################################
import unittest
def testParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['MessageID'] = 1
	params['RepeatIndicator'] = 1
	params['UserID'] = 1193046
	params['NavigationStatus'] = 3
	params['ROT'] = -2
	params['SOG'] = Decimal('101.9')
	params['PositionAccuracy'] = 1
	params['Position_longitude'] = Decimal('-122.16328055555556')
	params['Position_latitude'] = Decimal('37.424458333333334')
	params['COG'] = Decimal('34.5')
	params['TrueHeading'] = 41
	params['TimeStamp'] = 35
	params['RegionalReserved'] = 0
	params['Spare'] = 0
	params['RAIM'] = False
	params['syncstate'] = 2
	params['slotoffset'] = 1221

	return params

class Testposition(unittest.TestCase):
	'''Use testvalue tag text from each type to build test case the position message'''
	def testEncodeDecode(self):

		params = testParams()
		bits   = encode(params)
		r      = decode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['MessageID'],params['MessageID'])
		self.failUnlessEqual(r['RepeatIndicator'],params['RepeatIndicator'])
		self.failUnlessEqual(r['UserID'],params['UserID'])
		self.failUnlessEqual(r['NavigationStatus'],params['NavigationStatus'])
		self.failUnlessEqual(r['ROT'],params['ROT'])
		self.failUnlessAlmostEqual(r['SOG'],params['SOG'],1)
		self.failUnlessEqual(r['PositionAccuracy'],params['PositionAccuracy'])
		self.failUnlessAlmostEqual(r['Position_longitude'],params['Position_longitude'],5)
		self.failUnlessAlmostEqual(r['Position_latitude'],params['Position_latitude'],5)
		self.failUnlessAlmostEqual(r['COG'],params['COG'],3)
		self.failUnlessEqual(r['TrueHeading'],params['TrueHeading'])
		self.failUnlessEqual(r['TimeStamp'],params['TimeStamp'])
		self.failUnlessEqual(r['RegionalReserved'],params['RegionalReserved'])
		self.failUnlessEqual(r['Spare'],params['Spare'])
		self.failUnlessEqual(r['RAIM'],params['RAIM'])
		self.failUnlessEqual(r['syncstate'],params['syncstate'])
		self.failUnlessEqual(r['slotoffset'],params['slotoffset'])

def addMsgOptions(parser):
	parser.add_option('-d','--decode',dest='doDecode',default=False,action='store_true',
		help='decode a "position" AIS message')
	parser.add_option('-e','--encode',dest='doEncode',default=False,action='store_true',
		help='encode a "position" AIS message')
	parser.add_option('--RepeatIndicator-field', dest='RepeatIndicatorField',default=0,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--UserID-field', dest='UserIDField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--NavigationStatus-field', dest='NavigationStatusField',default=15,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--ROT-field', dest='ROTField',default=-128,metavar='int',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--SOG-field', dest='SOGField',default=Decimal('102.3'),metavar='udecimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--PositionAccuracy-field', dest='PositionAccuracyField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--Position_longitude-field', dest='Position_longitudeField',default=Decimal('181'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--Position_latitude-field', dest='Position_latitudeField',default=Decimal('91'),metavar='decimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--COG-field', dest='COGField',default=Decimal('360'),metavar='udecimal',type='string'
		,help='Field parameter value [default: %default]')
	parser.add_option('--TrueHeading-field', dest='TrueHeadingField',default=511,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--TimeStamp-field', dest='TimeStampField',default=60,metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--RAIM-field', dest='RAIMField',metavar='bool',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--syncstate-field', dest='syncstateField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--slotoffset-field', dest='slotoffsetField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')

############################################################
if __name__=='__main__':

	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options]",
		version="%prog "+__version__)

	parser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
		help='run the documentation tests')
	parser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
		help='run the unit tests')
	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
		help='Make the test output verbose')

	# FIX: remove nmea from binary messages.  No way to build the whole packet?
	# FIX: or build the surrounding msg 8 for a broadcast?
	typeChoices = ('binary','nmeapayload','nmea') # FIX: what about a USCG type message?
	parser.add_option('-t','--type',choices=typeChoices,type='choice',dest='ioType'
		,default='nmeapayload'
		,help='What kind of string to expect ('+', '.join(typeChoices)+') [default: %default]')


	outputChoices = ('std','html','csv' , 'kml','kml-full')
	parser.add_option('-T','--output-type',choices=outputChoices,type='choice',dest='outputType'
		,default='std'
		,help='What kind of string to output ('+', '.join(outputChoices)+') [default: %default]')

	parser.add_option('-o','--output',dest='outputFileName',default=None,
			  help='Name of the python file to write [default: stdout]')

	parser.add_option('-f','--fields',dest='fieldList',default=None, action='append',
			  choices=fieldList,
			  help='Which fields to include in the output.  Currently only for csv output [default: all]')

	parser.add_option('-p','--print-csv-field-list',dest='printCsvfieldList',default=False,action='store_true',
			  help='Print the field name for csv')

	addMsgOptions(parser)

	(options,args) = parser.parse_args()
	success=True

	if options.doctest:
		import os; print os.path.basename(sys.argv[0]), 'doctests ...',
		sys.argv= [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		import doctest
		numfail,numtests=doctest.testmod()
		if numfail==0: print 'ok'
		else: 
			print 'FAILED'
			success=False

	if not success: sys.exit('Something Failed')
	del success # Hide success from epydoc

	if options.unittest:
		sys.argv = [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		unittest.main()

	outfile = sys.stdout
	if None!=options.outputFileName:
		outfile = file(options.outputFileName,'w')


	if options.doEncode:
		# First make sure all non required options are specified
		if None==options.RepeatIndicatorField: parser.error("missing value for RepeatIndicatorField")
		if None==options.UserIDField: parser.error("missing value for UserIDField")
		if None==options.NavigationStatusField: parser.error("missing value for NavigationStatusField")
		if None==options.ROTField: parser.error("missing value for ROTField")
		if None==options.SOGField: parser.error("missing value for SOGField")
		if None==options.PositionAccuracyField: parser.error("missing value for PositionAccuracyField")
		if None==options.Position_longitudeField: parser.error("missing value for Position_longitudeField")
		if None==options.Position_latitudeField: parser.error("missing value for Position_latitudeField")
		if None==options.COGField: parser.error("missing value for COGField")
		if None==options.TrueHeadingField: parser.error("missing value for TrueHeadingField")
		if None==options.TimeStampField: parser.error("missing value for TimeStampField")
		if None==options.RAIMField: parser.error("missing value for RAIMField")
		if None==options.syncstateField: parser.error("missing value for syncstateField")
		if None==options.slotoffsetField: parser.error("missing value for slotoffsetField")
		msgDict={
			'MessageID': '1',
			'RepeatIndicator': options.RepeatIndicatorField,
			'UserID': options.UserIDField,
			'NavigationStatus': options.NavigationStatusField,
			'ROT': options.ROTField,
			'SOG': options.SOGField,
			'PositionAccuracy': options.PositionAccuracyField,
			'Position_longitude': options.Position_longitudeField,
			'Position_latitude': options.Position_latitudeField,
			'COG': options.COGField,
			'TrueHeading': options.TrueHeadingField,
			'TimeStamp': options.TimeStampField,
			'RegionalReserved': '0',
			'Spare': '0',
			'RAIM': options.RAIMField,
			'syncstate': options.syncstateField,
			'slotoffset': options.slotoffsetField,
		}

		bits = encode(msgDict)
		if 'binary'==options.ioType: print str(bits)
		elif 'nmeapayload'==options.ioType:
		        # FIX: figure out if this might be necessary at compile time
			print "bitLen",len(bits)
			bitLen=len(bits)
			if bitLen%6!=0:
			    bits = bits + BitVector(size=(6 - (bitLen%6)))  # Pad out to multiple of 6
			print "result:",binary.bitvectoais6(bits)[0]


		# FIX: Do not emit this option for the binary message payloads.  Does not make sense.
		elif 'nmea'==options.ioType: sys.exit("FIX: need to implement this capability")
		else: sys.exit('ERROR: unknown ioType.  Help!')

	if options.printCsvfieldList:
		# Make a csv separated list of fields that will be displayed for csv
		if None == options.fieldList: options.fieldList = fieldList
		import StringIO
		buf = StringIO.StringIO()
		for field in options.fieldList:
			buf.write(field+',')
		result = buf.getvalue()
		if result[-1] == ',': print result[:-1]
		else: print result

	if options.doDecode:
		for msg in args:
			bv = None
			if    'binary'    == options.ioType: bv = BitVector(bitstring=msg)
			elif 'nmeapayload'== options.ioType: bv = binary.ais6tobitvec(msg)
			elif 'nmea'       == options.ioType: bv = binary.ais6tobitvec(msg.split(',')[5])
			else: sys.exit('ERROR: unknown ioType.  Help!')

			printFields(decode(bv),out=outfile,format=options.outputType,fieldList=options.fieldList)
