#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2007-01-02 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Get the name and mmsi (User ID) for each msg 5 identification message.

This program requires that the nmea strings be sorted such that the
2nd part of the message comes directly after the first.  You can do
this by grepping for the USCG N-AIS station name.  This will prevent
interleaving of msg 5 nmea strings.

@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{pyproj<http:///>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: under development
@license: GPL
'''

import sys, os

# Can decode messages 1,2,3 will any of the three codecs

import binary, ais_msg_1, aisstring
#from BitVector import BitVector

from pyproj import Proj
import math

def dist (lon1, lat1, lon2, lat2):
    dx = (lon1-lon2)
    dy = (lat1-lat2)
    return math.sqrt(dx*dx + dy*dy)

def getPosition(logfile,outfile,minDist=None):


    # FIX: use the right utm zone.  14 is the central US so it will kind of work
    params={'proj':'utm','zone':14} #int(options.zone)}
    proj=None
    if minDist!=None:
	proj = Proj(params)

    positions = {} # Last recoded ship position

    #print 'mindist:',minDist

    for line in logfile: 
	fields = line.split(',')
	if '1'!=fields[2]: # Must be the start of a sequence
	    continue
	bv = binary.ais6tobitvec(fields[5][:39]) # Hacked for speed

	timestamp = fields[-1].strip()
	mmsi = str(ais_msg_1.decodeUserID(bv))
	lon = ais_msg_1.decodePosition_longitude(bv)
	lat = ais_msg_1.decodePosition_latitude(bv)

	d = None
	if mmsi not in positions:
	    positions[mmsi]=(lon,lat)
	elif minDist != None:
	    lonOld,latOld = positions[mmsi]
	    oldUTM = proj(lonOld,latOld)
	    newUTM = proj(lon,lat)
	    d = dist(oldUTM[0],oldUTM[1],newUTM[0],newUTM[1])
	    if str(d)=='nan':
		continue #pass  # FIX: Print but do not save nan values???
	    elif d < minDist:
		continue
	    else:
		positions[mmsi]=(lon,lat)

	lon = str(lon)
	lat = str(lat)

	if len(mmsi)<9: mmsi += ' '*(9-len(mmsi))

	fLen = 12 # field length ... how much space

	if len(lon)>fLen: lon = lon[:fLen]
	if len(lon)<fLen: lon += ' '*(fLen-len(lon))

	if len(lat)>fLen: lat = lat[:fLen]
	if len(lat)<fLen: lat += ' '*(fLen-len(lat))

	#outfile.write(timestamp+' '+str(mmsi)+' '+lon+' '+lat+' -- '+str(d)+'\n')
	outfile.write(timestamp+' '+str(mmsi)+' '+lon+' '+lat+'\n')

if __name__=='__main__':
    from optparse import OptionParser
    parser = OptionParser(usage="%prog [options] logfile1 [logfile2 logfile3 ...] ", version="%prog "+__version__)
    parser.add_option('-o','--output',dest='outputFileName',default=None,
		      help='Name of the python file to write [default: stdout]')
    parser.add_option('-m','--min-dist',dest='minDist',default=None,type='float',
		      help='minimum distance to move before output a new position in meters [default: None]')

    # FIX: add option that a max time between positions to pass if the ship is not moving

    (options,args) = parser.parse_args()
    
    outfile = sys.stdout
    if None!=options.outputFileName:
	print 'outfilename=',options.outputFileName
	outfile = file(options.outputFileName,'w')
    if 0==len(args):
	getPosition(sys.stdin,outfile,options.minDist)
    else: 
	for filename in args:
	    getPosition(file(filename),outfile,options.minDist)

