#!/usr/bin/env python

__version__ = '$Revision: 4791 $'.split()[1]
__date__ = '$Date: 2007-01-18 $'.split()[1]
__author__ = 'xmlbinmsg'

__doc__='''

Autogenerated python functions to serialize/deserialize binary messages.

Generated by: ./aisxmlbinmsg2py.py

Need to then wrap these functions with the outer AIS packet and then
convert the whole binary blob to a NMEA string.  Those functions are
not currently provided in this file.

serialize: python to ais binary
deserialize: ais binary to python

The generated code uses translators.py, binary.py, and aisstring.py
which should be packaged with the resulting files.


@requires: U{epydoc<http://epydoc.sourceforge.net/>} > 3.0alpha3
@requires: U{BitVector<http://cheeseshop.python.org/pypi/BitVector>}

@author: '''+__author__+'''
@version: ''' + __version__ +'''
@var __date__: Date of last svn commit
@undocumented: __version__ __author__ __doc__ parser
@status: under development
@license: Generated code has no license
'''

import sys
from decimal import Decimal
from BitVector import BitVector

import binary, aisstring

# FIX: check to see if these will be needed
TrueBV  = BitVector(bitstring="1")
"Why always rebuild the True bit?  This should speed things up a bunch"
FalseBV = BitVector(bitstring="0")
"Why always rebuild the False bit?  This should speed things up a bunch"


fieldList = [
	'dac',
	'fid',
	'reserved',
	'MessageID',
	'BinaryData',
]

def encode(params, validate=False):
	'''Create a sls_header binary message payload to pack into an AIS Msg sls_header.

	Fields in params:
	  - dac(uint): Designated Area Code 366 for US
	  - fid(uint): Functional Id (field automatically set to "1")
	  - reserved(uint): say what? (field automatically set to "0")
	  - MessageID(uint): Binary message indentifier
	  - BinaryData(binary):  FIX: make this consume the rest! 
	@param params: Dictionary of field names/values.  Throws a ValueError exception if required is missing
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: BitVector
	@return: encoded binary message (for binary messages, this needs to be wrapped in a msg 8
	@note: The returned bits may not be 6 bit aligned.  It is up to you to pad out the bits.
	'''

	bvList = []
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['dac']),10))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=1),6))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=0),2))
	bvList.append(binary.setBitVectorSize(BitVector(intVal=params['MessageID']),6))
	bvList.append(params['BinaryData'])

	return binary.joinBV(bvList)

def decode(bv, validate=False):
	'''Unpack a sls_header message 

	Fields in params:
	  - dac(uint): Designated Area Code 366 for US
	  - fid(uint): Functional Id (field automatically set to "1")
	  - reserved(uint): say what? (field automatically set to "0")
	  - MessageID(uint): Binary message indentifier
	  - BinaryData(binary):  FIX: make this consume the rest! 
	@type bv: BitVector
	@param bv: Bits defining a message
	@param validate: Set to true to cause checking to occur.  Runs slower.  FIX: not implemented.
	@rtype: dict
	@return: params
	'''

	#Would be nice to check the bit count here..
	#if validate:
	#	assert (len(bv)==FIX: SOME NUMBER)
	r = {}
	r['dac']=int(bv[0:10])
	r['fid']=1
	r['reserved']=0
	r['MessageID']=int(bv[18:24])
	r['BinaryData']=bv[24:]
	return r

def decodedac(bv, validate=False):
	return int(bv[0:10])

def decodefid(bv, validate=False):
	return 1

def decodereserved(bv, validate=False):
	return 0

def decodeMessageID(bv, validate=False):
	return int(bv[18:24])

def decodeBinaryData(bv, validate=False):
	return bv[24:]


def printHtml(params, out=sys.stdout):
		out.write("<h3>sls_header<h3>\n")
		out.write("<table border=\"1\">\n")
		out.write("<tr bgcolor=\"orange\">\n")
		out.write("<th align=\"left\">Field Name</th>\n")
		out.write("<th align=\"left\">Type</th>\n")
		out.write("<th align=\"left\">Value</th>\n")
		out.write("<th align=\"left\">Value in Lookup Table</th>\n")
		out.write("<th align=\"left\">Units</th>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>dac</td>\n")
		out.write("<td>uint</td>\n")
		if 'dac' in params:
			out.write("	<td>"+str(params['dac'])+"</td>\n")
			out.write("	<td>"+str(params['dac'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>fid</td>\n")
		out.write("<td>uint</td>\n")
		if 'fid' in params:
			out.write("	<td>"+str(params['fid'])+"</td>\n")
			out.write("	<td>"+str(params['fid'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>reserved</td>\n")
		out.write("<td>uint</td>\n")
		if 'reserved' in params:
			out.write("	<td>"+str(params['reserved'])+"</td>\n")
			out.write("	<td>"+str(params['reserved'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>MessageID</td>\n")
		out.write("<td>uint</td>\n")
		if 'MessageID' in params:
			out.write("	<td>"+str(params['MessageID'])+"</td>\n")
			out.write("	<td>"+str(params['MessageID'])+"</td>\n")
		out.write("</tr>\n")
		out.write("\n")
		out.write("<tr>\n")
		out.write("<td>BinaryData</td>\n")
		out.write("<td>binary</td>\n")
		if 'BinaryData' in params:
			out.write("	<td>"+str(params['BinaryData'])+"</td>\n")
			out.write("	<td>"+str(params['BinaryData'])+"</td>\n")
		out.write("</tr>\n")
		out.write("</table>\n")

def printFields(params, out=sys.stdout, format='std', fieldList=None):
	'''Print a BinaryData message to stdout.

	Fields in params:
	  - dac(uint): Designated Area Code 366 for US
	  - fid(uint): Functional Id (field automatically set to "1")
	  - reserved(uint): say what? (field automatically set to "0")
	  - MessageID(uint): Binary message indentifier
	  - BinaryData(binary):  FIX: make this consume the rest! 
	@param params: Dictionary of field names/values.  
	@param out: File like object to write to
	@rtype: stdout
	@return: text to out
	'''

	if 'std'==format:
		out.write("BinaryData:\n")
		if 'dac' in params: out.write("	dac:         "+str(params['dac'])+"\n")
		if 'fid' in params: out.write("	fid:         "+str(params['fid'])+"\n")
		if 'reserved' in params: out.write("	reserved:    "+str(params['reserved'])+"\n")
		if 'MessageID' in params: out.write("	MessageID:   "+str(params['MessageID'])+"\n")
		if 'BinaryData' in params: out.write("	BinaryData:  "+str(params['BinaryData'])+"\n")
	elif 'csv'==format:
		if None == options.fieldList:
			options.fieldList = fieldList
		needComma = False;
		for field in fieldList:
			if needComma: out.write(',')
			needComma = True
			if field in params:
				out.write(str(params[field]))
			# else: leave it empty
		out.write("\n")
	elif 'html'==format:
		printHtml(params,out)
	else: 
		print "ERROR: unknown format:",format
		assert False

	return # Nothing to return



######################################################################
# UNIT TESTING
######################################################################
import unittest
def testParams():
	'''Return a params file base on the testvalue tags.
	@rtype: dict
	@return: params based on testvalue tags
	'''
	params = {}
	params['dac'] = 366
	params['fid'] = 1
	params['reserved'] = 0
	params['MessageID'] = 3
	params['BinaryData'] = BitVector(bitstring='0')

	return params

class Testsls_header(unittest.TestCase):
	'''Use testvalue tag text from each type to build test case the sls_header message'''
	def testEncodeDecode(self):

		params = testParams()
		bits   = encode(params)
		r      = decode(bits)

		# Check that each parameter came through ok.
		self.failUnlessEqual(r['dac'],params['dac'])
		self.failUnlessEqual(r['fid'],params['fid'])
		self.failUnlessEqual(r['reserved'],params['reserved'])
		self.failUnlessEqual(r['MessageID'],params['MessageID'])
		self.failUnlessEqual(r['BinaryData'],params['BinaryData'])

def addMsgOptions(parser):
	parser.add_option('-d','--decode',dest='doDecode',default=False,action='store_true',
		help='decode a "sls_header" AIS message')
	parser.add_option('-e','--encode',dest='doEncode',default=False,action='store_true',
		help='encode a "sls_header" AIS message')
	parser.add_option('--dac-field', dest='dacField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--MessageID-field', dest='MessageIDField',metavar='uint',type='int'
		,help='Field parameter value [default: %default]')
	parser.add_option('--BinaryData-field', dest='BinaryDataField',metavar='binary',type='string'
		,help='Field parameter value [default: %default]')

############################################################
if __name__=='__main__':

	from optparse import OptionParser
	parser = OptionParser(usage="%prog [options]",
		version="%prog "+__version__)

	parser.add_option('--doc-test',dest='doctest',default=False,action='store_true',
		help='run the documentation tests')
	parser.add_option('--unit-test',dest='unittest',default=False,action='store_true',
		help='run the unit tests')
	parser.add_option('-v','--verbose',dest='verbose',default=False,action='store_true',
		help='Make the test output verbose')

	# FIX: remove nmea from binary messages.  No way to build the whole packet?
	# FIX: or build the surrounding msg 8 for a broadcast?
	typeChoices = ('binary','nmeapayload','nmea') # FIX: what about a USCG type message?
	parser.add_option('-t','--type',choices=typeChoices,type='choice',dest='ioType'
		,default='nmeapayload'
		,help='What kind of string to expect ('+', '.join(typeChoices)+') [default: %default]')


	outputChoices = ('std','html','csv' )
	parser.add_option('-T','--output-type',choices=outputChoices,type='choice',dest='outputType'
		,default='std'
		,help='What kind of string to output ('+', '.join(outputChoices)+') [default: %default]')

	parser.add_option('-o','--output',dest='outputFileName',default=None,
			  help='Name of the python file to write [default: stdout]')

	parser.add_option('-f','--fields',dest='fieldList',default=None, action='append',
			  choices=fieldList,
			  help='Which fields to include in the output.  Currently only for csv output [default: all]')

	parser.add_option('-p','--print-csv-field-list',dest='printCsvfieldList',default=False,action='store_true',
			  help='Print the field name for csv')

	addMsgOptions(parser)

	(options,args) = parser.parse_args()
	success=True

	if options.doctest:
		import os; print os.path.basename(sys.argv[0]), 'doctests ...',
		sys.argv= [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		import doctest
		numfail,numtests=doctest.testmod()
		if numfail==0: print 'ok'
		else: 
			print 'FAILED'
			success=False

	if not success: sys.exit('Something Failed')
	del success # Hide success from epydoc

	if options.unittest:
		sys.argv = [sys.argv[0]]
		if options.verbose: sys.argv.append('-v')
		unittest.main()

	outfile = sys.stdout
	if None!=options.outputFileName:
		outfile = file(options.outputFileName,'w')


	if options.doEncode:
		# First make sure all non required options are specified
		if None==options.dacField: parser.error("missing value for dacField")
		if None==options.MessageIDField: parser.error("missing value for MessageIDField")
		if None==options.BinaryDataField: parser.error("missing value for BinaryDataField")
		msgDict={
			'dac': options.dacField,
			'fid': '1',
			'reserved': '0',
			'MessageID': options.MessageIDField,
			'BinaryData': options.BinaryDataField,
		}

		bits = encode(msgDict)
		if 'binary'==options.ioType: print str(bits)
		elif 'nmeapayload'==options.ioType:
		        # FIX: figure out if this might be necessary at compile time
			print "bitLen",len(bits)
			bitLen=len(bits)
			if bitLen%6!=0:
			    bits = bits + BitVector(size=(6 - (bitLen%6)))  # Pad out to multiple of 6
			print "result:",binary.bitvectoais6(bits)[0]


		# FIX: Do not emit this option for the binary message payloads.  Does not make sense.
		elif 'nmea'==options.ioType: sys.exit("FIX: need to implement this capability")
		else: sys.exit('ERROR: unknown ioType.  Help!')

	if options.printCsvfieldList:
		# Make a csv separated list of fields that will be displayed for csv
		if None == options.fieldList: options.fieldList = fieldList
		import StringIO
		buf = StringIO.StringIO()
		for field in options.fieldList:
			buf.write(field+',')
		result = buf.getvalue()
		if result[-1] == ',': print result[:-1]
		else: print result

	if options.doDecode:
		for msg in args:
			bv = None
			if    'binary'    == options.ioType: bv = BitVector(bitstring=msg)
			elif 'nmeapayload'== options.ioType: bv = binary.ais6tobitvec(msg)
			elif 'nmea'       == options.ioType: bv = binary.ais6tobitvec(msg.split(',')[5])
			else: sys.exit('ERROR: unknown ioType.  Help!')

			printFields(decode(bv),out=outfile,format=options.outputType,fieldList=options.fieldList)
